/**
 * 订单相关
 */
import moment = require("moment");
import { COSTTYPE, INVOICESTATUS, ISPAYENUM, MEMBERLEVEL, MEMBERTYPE, ORDERSTATE, PAYMENTTYPE, RECEIPTCONFIRMATION } from "../../config/enum";
import { generateOrderId, successResult } from "../../tools/system";
import { changeEnumValue, eccEnumValue } from "../../util/verificationEnum";
import { addOneData } from "../../data/add";
import { TABLEENUM } from "../../data/models/model";
import { find, findCount, findOnce, findToPage } from "../../data/select";
import { extractData } from "../../util/piecemeal";
import { BizError } from "../../util/bizError";
import { ERRORENUM } from "../../config/errorEnum";
import { updateOneData } from "../../data/update";


/**
 * 对内使用  创建一条订单
 * @param userId 会员id
 * @param unitName 单位名称
 * @param orderStartMs 订单开始时间
 * @param memberLv 会员等级 MEMBERLEVEL
 * @param memberType 会员类型 MEMBERTYPE
 * @param isFirst 是否首次缴费
 * @returns 
 */
export async function createOrder(userId:string, unitName:string, orderStartMs:number, memberLv:number, memberType:number, isFirst:boolean) {
    /**计算订单周期 */
    const EndMs = moment(orderStartMs).add(1, 'years').valueOf();
    /**订单id */
    const OrderId = generateOrderId(userId);
    /**计算会费 */
    let money = 0;
    if (memberLv == MEMBERLEVEL.专家会员) {

    } else {

    }
    let addInfo = {
        id:OrderId,
        orderCycleStart:orderStartMs,
        orderCycleEnd:EndMs,
        unitName,
        money,
        paymentMethod:0,
        memberId:userId,
        memberCategory:memberType,
        paymentNum:0,
        invoiceAdd:'',
        state:ORDERSTATE.未支付,
        isFirst
    };

    // await addOneData(TABLEENUM.订单表, addInfo);
    return true;
}


/**
 * 初始订单
 */
export async function firstOrderList({unitName, joinTime, memberType, orderState, pageNumber}) {
    let findParam:any = { isFirst:true };
    if (unitName) findParam.unitName = {"$regex":unitName};
    if (memberType) findParam.memberCategory = memberType;
    if (orderState) findParam.state = orderState;

    let selectFile = ["unitName", "loginId", "phone", "orderCycleStart", "orderCycleEnd", "memberCategory", "state"];
    let dbList = await findToPage(TABLEENUM.订单表, findParam, selectFile, pageNumber);
    let dataCount = await findCount(TABLEENUM.订单表, findParam);
    let dataList = [];
    let itemFile = ["unitName", "loginId", "phone"];
    dbList.forEach(info =>  {
        let itemData:any = extractData(info, itemFile);
        itemData.joinTime = moment(info.orderCycleStart).format("YYYY-MM-DD");
        itemData.memberType = changeEnumValue(MEMBERTYPE, itemData.memberCategory);
        itemData.paymentMethod = changeEnumValue(MEMBERTYPE, itemData.paymentMethod);
        itemData.cycle = `${moment(info.orderCycleStart).format("YYYY-MM-DD")}至${moment(info.orderCycleEnd).format("YYYY-MM-DD")}`
        dataList.push(itemData);
    });

    return {dataList, dataCount}
}


/**
 * 财务核对列表
 */
export async function paidList({unitName, paymentType, costType, pageNumber }) {
    eccEnumValue("财务核对列表", "支付类型", PAYMENTTYPE, paymentType);
    eccEnumValue("财务核对列表", "会费类别", COSTTYPE, costType );

    let findParam:any = { isFirst:true };
    if (unitName) findParam.unitName = {"$regex":unitName};
    if (paymentType) findParam.paymentMethod = paymentType;
    if (costType) findParam.isFirst = costType == COSTTYPE.首次;

    let selectFile = ["id", "unitName", "orderCycleStart", "orderCycleEnd", "money", "paymentMethod", "memberCategory", "isFirst", "paymentNum" ];
    let dbList = await findToPage(TABLEENUM.订单表, findParam, selectFile, pageNumber);
    let dataCount = await findCount(TABLEENUM.订单表, findParam);

    let dataList = [];

    let itemFile = ["unitName", "money", "paymentNum"];
    dbList.forEach(info =>  {
        let itemData:any = extractData(info, itemFile);
        itemData.joinTime = moment(info.orderCycleStart).format("YYYY-MM-DD");
        itemData.memberType = changeEnumValue(MEMBERTYPE, itemData.memberCategory);
        itemData.paymentMethod = changeEnumValue(MEMBERTYPE, itemData.paymentMethod);
        itemData.costType = info.isFirst == COSTTYPE.首次 ? "首次" : "续费";
        itemData.cycle = `${moment(info.orderCycleStart).format("YYYY-MM-DD")}至${moment(info.orderCycleEnd).format("YYYY-MM-DD")}`
        dataList.push(itemData);
    });

    return {dataList, dataCount};
}


/**
 * 待开发票和已开发票
 */
export async function billStateList({unitName, paymentType, isPay, pageNumber, invoiceStatus}) {
    eccEnumValue("发票列表", "支付类型", PAYMENTTYPE, paymentType);
    eccEnumValue("发票列表", "是否支付", ISPAYENUM, isPay );
    eccEnumValue("发票列表", "发票类型", INVOICESTATUS, invoiceStatus )

    let findParam:any = { isFirst:true };
    if (unitName) findParam.unitName = {"$regex":unitName};
    if (paymentType) findParam.paymentMethod = paymentType;
    if (isPay) findParam.state = isPay

    let selectFile = ["id", "unitName", "orderCycleStart", "orderCycleEnd", "money", "paymentMethod", "memberCategory", "state", "paymentNum" ];
    let dbList = await findToPage(TABLEENUM.订单表, findParam, selectFile, pageNumber);
    let dataCount = await findCount(TABLEENUM.订单表, findParam);

    let dataList = [];

    let itemFile = ["unitName", "money", "paymentNum"];
    dbList.forEach(info =>  {
        let itemData:any = extractData(info, itemFile);
        itemData.joinTime = moment(info.orderCycleStart).format("YYYY-MM-DD");
        itemData.memberType = changeEnumValue(MEMBERTYPE, itemData.memberCategory);
        itemData.paymentMethod = changeEnumValue(MEMBERTYPE, itemData.paymentMethod);
        itemData.cycle = `${moment(info.orderCycleStart).format("YYYY-MM-DD")}至${moment(info.orderCycleEnd).format("YYYY-MM-DD")}`
        dataList.push(itemData);
    });

    return {dataList, dataCount};
}


/**
 * 待支付-通过审核
 * @param param0 
 */
export async function checkOrder({id}) {
    let orderInfo = await findOnce(TABLEENUM.订单表, {id});
    if (!orderInfo || !orderInfo.id ) throw new BizError(ERRORENUM.目标数据不存在);

    if (orderInfo.firstExamine) throw new BizError(ERRORENUM.重复提交);

    await updateOneData(TABLEENUM.订单表, {id}, {firstExamine:true});

    return successResult();
}


/**
 * 收款确认通过
 * @param param0 
 */
export async function confirmReceiptPass({id}) {
    let orderInfo = await findOnce(TABLEENUM.订单表, {id});
    if (!orderInfo || !orderInfo.id ) throw new BizError(ERRORENUM.目标数据不存在);

    if (orderInfo.confirmReceipt != RECEIPTCONFIRMATION.待确认) throw new BizError(ERRORENUM.重复提交, `收款确认通过时 状态处于${orderInfo.confirmReceipt}`);

    await updateOneData(TABLEENUM.订单表, {id}, {confirmReceipt:RECEIPTCONFIRMATION.收款确认});

    return successResult();
}


/**
 * 收款确认失败
 * @param param0 
 */
export async function confirmReceiptOut({id}) {
    let orderInfo = await findOnce(TABLEENUM.订单表, {id});
    if (!orderInfo || !orderInfo.id ) throw new BizError(ERRORENUM.目标数据不存在);

    if (orderInfo.confirmReceipt != RECEIPTCONFIRMATION.待确认) throw new BizError(ERRORENUM.重复提交, `收款确认通过时 状态处于${orderInfo.confirmReceipt}`);

    await updateOneData(TABLEENUM.订单表, {id}, {confirmReceipt:RECEIPTCONFIRMATION.退回});

    return successResult();
}


/**
 * 个人页待订单列表
 * @param param0 
 */
export async function orderList({userId, type}) {
    eccEnumValue("个人页待订单列表", "类型", ISPAYENUM, type);
    let findParam:any = {};
    if (type == ORDERSTATE.已支付) findParam.state = ORDERSTATE.已支付;
    else if (type == ORDERSTATE.未支付) findParam.state = ORDERSTATE.未支付;

    const SelectFile = ["orderCycleStart", "orderCycleEnd", "money", "state", "id"];
    let dbList = await find(TABLEENUM.订单表, findParam, SelectFile);

    let dataList = [];
    dbList.forEach(info => {
        let {orderCycleStart, orderCycleEnd, money, state, id} = info;
        let moneyStr = state == ORDERSTATE.已支付 ? "已缴金额" : "待缴金额"; 
        dataList.push({
            cycle:`${moment(orderCycleStart).format("YYYY-MM-DD")}至${moment(orderCycleEnd).format("YYYY-MM-DD")}`,
            money:`${moneyStr}:${money}`,
            id
        });
    });

    return {dataList}
}